@@ -1,8 +1,6 @@
 symm.huber <- function(X, qg=0.9, init=NULL, eps = 1e-06, maxiter = 100, na.action=na.fail)
     {
     X <- na.action(X)
-    if (!all(sapply(X, is.numeric))) 
-        stop("'X' must be numeric")
     X <- as.matrix(X)
     CNAMES <- colnames(X)
     p <- ncol(X)
@@ -11,11 +9,10 @@ symm.huber <- function(X, qg=0.9, init=NULL, eps = 1e-06, maxiter = 100, na.acti
     sigma.square <- 2 * pchisq(c.square / 2, p + 2) + (c.square / p) * (1 - qg)
     
     if (p<2) stop("'X' must be at least bivariate")  
-    
-    X<-pair.diff(X)
+
     n<-dim(X)[1]
 
-    if (is.numeric(init)) V.0 <- init else V.0<- crossprod(X) / n
+    if (is.numeric(init)) V.0 <- init else V.0<- cov(X)
  
     iter<-0
     differ<-Inf
@@ -27,10 +24,8 @@ symm.huber <- function(X, qg=0.9, init=NULL, eps = 1e-06, maxiter = 100, na.acti
              stop("maxiter reached without convergence")
             }
         iter<-iter+1
-        d2<- mahalanobis(X,0,V.0)
-        w2<-ifelse(d2<=c.square, 1/sigma.square, (c.square/d2)/sigma.square)
-        w.X <- sqrt(w2) * X
-        V.new<- (1/n) * crossprod(w.X)
+        
+        V.new<- SSCov.hub(X,solve(V.0),c.square,sigma.square)
         differ <- ICS:::frobenius.norm(V.new-V.0)    
         V.0<-V.new
         }
@@ -38,3 +33,12 @@ symm.huber <- function(X, qg=0.9, init=NULL, eps = 1e-06, maxiter = 100, na.acti
     rownames(V.new) <- CNAMES
     V.new
     }
+
+# Internal function for symm.huber
+
+SSCov.hub<-function(X,V,c.s,sig.s)
+{
+X<-as.matrix(X)
+d<-dim(X)
+matrix(.C("symm_huber", as.double(X), as.double(V), as.integer(d), as.double(c.s), as.double(sig.s),res=double(d[2]^2))$res, ncol=d[2],byrow=T)/(d[1]*(d[1]-1)/2)
+}
